@@ -3,17 +3,18 @@
 
 function Events1() { // demo 
 
-	function on_page_loaded(task) { 
-		
+	function on_page_loaded(task) {   
+		$("title").text(task.item_caption);
 		$("#title").text(task.item_caption);
+		
 		if (task.safe_mode) {
 			$("#user-info").text(task.user_info.role_name + ' ' + task.user_info.user_name);
 			$('#log-out')
-			.show()
+			.show() 
 			.click(function(e) {
-				e.preventDefault(); 
+				e.preventDefault();
 				task.logout();
-			});
+			}); 
 		}
 	
 		$("#taskmenu").show();
@@ -43,12 +44,12 @@ function Events1() { // demo
 								)
 							);
 						}
-					}
+					} 
 				}
 			}
 		));
 		$('#submenu').on('click', 'a', (function(e) {
-			var item = $(this).data('item');
+			var item = $(this).data('item'); 
 			e.preventDefault();
 			$("#submenu > li" ).removeClass('active');
 			$(this).parent().addClass('active');
@@ -63,18 +64,15 @@ function Events1() { // demo
 		$("#menu").append($('<li></li>').append($('<a href="#">About</a>').click(function(e) {
 			e.preventDefault();
 			task.message(
-				'<a href="http://jam-py.com/" target="_blank"><h3>Jam.py</h3></a>' +
-				'<h3>Demo application</h3>' +
-				' with <a href="http://chinookdatabase.codeplex.com/" target="_blank">Chinook Database</a>' +
-				'<p>by Andrew Yushev</p>' +
-				'<p>2016</p>',
-				{title: 'Jam.py framework', margin: 0, text_center: true, buttons: {"OK": undefined}, center_buttons: true}
+				task.templates.find('.about'),
+				{title: 'Jam.py framework', margin: 0, text_center: true, 
+					buttons: {"OK": undefined}, center_buttons: true}
 			);
 		})));
-		$("#menu").append($('<li><a href="http://jam-py.com/" target="_blank">Jam.py</a></li>'));	
+		$("#menu").append($('<li><a href="http://jam-py.com/" target="_blank">Jam.py</a></li>'));	 
 		
-		$('#menu').children(":first").find('a').click();
-		$('#submenu').children(":first").find('a').click();	
+		$('#menu').children(":first").find('a').click(); 
+		$('#submenu').children(":first").find('a').click();
 		
 		// $(document).ajaxStart(function() { $("html").addClass("wait"); });
 		// $(document).ajaxStop(function() { $("html").removeClass("wait"); });
@@ -138,7 +136,7 @@ function Events1() { // demo
 	
 	
 	function on_view_form_close_query(item) {
-		item.close();
+	//	item.close();
 	}
 	
 	function on_edit_form_created(item) {
@@ -265,8 +263,8 @@ function Events1() { // demo
 					height = 200;
 				}
 				table.height(height);
-				item.update_controls();
-				item.each_detail(function(d) {
+				table.item.update_controls();
+				table.item.each_detail(function(d) {
 					d.update_controls();
 				});
 			}
@@ -289,7 +287,7 @@ function Events1() { // demo
 						height = 200;
 					}
 					table.height(height);
-					item.update_controls();
+					table.item.update_controls();
 				}			
 			}
 		});
@@ -330,13 +328,34 @@ function Events1() { // demo
 
 task.events.events1 = new Events1();
 
+function Events3() { // demo.journals 
+
+	function on_view_form_created(item) {
+		item.view_form.find("#filter-btn").click(function() {item.create_filter_form()});	
+		if (!item.on_filters_applied && item.view_form) {
+			item.on_filters_applied = function() {
+				item.view_form.find("#filter-text").text(item.get_filter_text());		
+			};
+		}
+		setTimeout(
+			function() {
+				item.view_form.find('.dbtable.' + item.item_name + ' .inner-table').focus();
+			},
+			100
+		);	
+	}
+	this.on_view_form_created = on_view_form_created;
+}
+
+task.events.events3 = new Events3();
+
 function Events2() { // demo.catalogs 
 
 	function on_view_form_created(item) {
 		var timeOut,
 			search;
 		if (item.default_field) {
-			if (item.lookup_field && item.lookup_field.value) {
+			if (item.lookup_field && item.lookup_field.value && !item.lookup_field.multi_select) {
 				item.view_form.find(".view-title #title-left")
 					.append($('<p><a href="#" id="cur_value" tabindex="-1">' + item.lookup_field.lookup_text + '</a></p>'))
 					.css('padding-top', '12px');
@@ -371,7 +390,7 @@ function Events2() { // demo.catalogs
 			});
 			item.view_form.on('keydown', function(e) {
 				var code = e.which;
-				if (isCharCode(code) || code === 32 || code === 8) {
+				if (isCharCode(code) || code === 8) {				
 					if (!search.is(":focus")) {
 						if (code !== 8) {
 							search.val('');
@@ -393,8 +412,14 @@ function Events2() { // demo.catalogs
 	}
 	
 	function on_view_form_shown(item) {
-		setTimeout(function() {
-				item.view_form.find(".view-title input").focus();
+		setTimeout(
+			function() {
+				if (item.default_field) {
+					item.view_form.find(".view-title input").focus();
+				}
+				else {
+					item.view_form.find('.dbtable.' + item.item_name + ' .inner-table').focus();
+				}
 			},
 			100
 		);	
@@ -429,21 +454,24 @@ function Events16() { // demo.journals.invoices
 		};
 	}
 	
-	function on_view_form_created(item) {
-		item.view_form.find("#filter-btn").click(function() {item.create_filter_form()});	
-	
-		item.invoice_table.create_table(item.view_form.find(".view-detail"),
-			{height: 200 - 4, dblclick_edit: false, column_width: {"track": "60%"}});
+	function init_inputs(item, input_options) {
+		input_options.col_count = 2;	
+		// input_options.col_count = 4;
+		// input_options.label_on_top = true;
 	}
 	
-	function on_view_form_shown(item) {
-		item.view_form.find(".dbtable.invoices .inner-table").focus();
+	function on_view_form_created(item) {
+		item.invoice_table.create_table(item.view_form.find(".view-detail"),
+			{
+				height: 200 - 4, 
+				dblclick_edit: false, 
+				column_width: {'track': '25%', 'album': '25%', 'artists': '10%'}
+			});
 	}
 	
 	function on_filters_applied(item) {
 		if (item.view_form) {
-			item.view_form.find(".view-title #title-right")
-				.html('<h5 class="pull-right">' + item.get_filter_text() + '<h5>');
+			item.view_form.find("#filter-text").text(item.get_filter_text());
 			calc_footer(item);
 		}
 	}
@@ -466,10 +494,6 @@ function Events16() { // demo.journals.invoices
 		);
 	}
 	
-	function init_inputs(item, input_options) {
-		input_options.col_count = 2;
-	}
-	
 	function on_edit_form_created(item) {
 		item.edit_options.width = 1050;
 		item.invoice_table.create_table(item.edit_form.find(".edit-detail"),
@@ -479,7 +503,7 @@ function Events16() { // demo.journals.invoices
 				editable: true,
 				editable_fields: ['track', 'quantity'],
 				sortable: true,
-				column_width: {"track": "60%"}
+				column_width: {'track': '25%', 'album': '25%', 'artists': '10%'}
 			});
 		item.edit_form.find("#new-btn")
 			.on('click.task', function() { item.invoice_table.append_record() });
@@ -489,7 +513,7 @@ function Events16() { // demo.journals.invoices
 			.on('click.task', function() { item.invoice_table.delete_record() });
 	}
 	
-	function on_get_field_text(field) {
+	function on_field_get_text(field) {
 		if (field.field_name === 'customer' && field.value) {
 			return field.owner.firstname.lookup_text + ' ' + field.lookup_text;
 		}
@@ -497,7 +521,7 @@ function Events16() { // demo.journals.invoices
 	
 	function on_field_changed(field, lookup_item) {
 		if (field.field_name === 'taxrate') {
-			calculate(field.owner, true);
+			calculate(field.owner, true); 
 		}
 	}
 	
@@ -575,13 +599,12 @@ function Events16() { // demo.journals.invoices
 	}
 	this.on_after_append = on_after_append;
 	this.init_table = init_table;
+	this.init_inputs = init_inputs;
 	this.on_view_form_created = on_view_form_created;
-	this.on_view_form_shown = on_view_form_shown;
 	this.on_filters_applied = on_filters_applied;
 	this.calc_footer = calc_footer;
-	this.init_inputs = init_inputs;
 	this.on_edit_form_created = on_edit_form_created;
-	this.on_get_field_text = on_get_field_text;
+	this.on_field_get_text = on_field_get_text;
 	this.on_field_changed = on_field_changed;
 	this.calc_total = calc_total;
 	this.calculate = calculate;
@@ -595,26 +618,27 @@ task.events.events16 = new Events16();
 function Events15() { // demo.catalogs.tracks 
 
 	function init_table(item, options) {
-	//	options.sortable = false;
 		options.row_line_count = 2;
 		options.expand_selected_row = 3;
-	}
-	
-	function on_before_post(item) {
-	   item.track.value = item.name.value;
-	   if (item.album.value) {
-		   item.track.value += '; album: ' + item.album.display_text;
-	   }
-	   if (item.composer.value) {
-		   item.track.value += '; composer: ' + item.composer.display_text;
-	   }
+		options.column_width = {'artist': '7%'};
 	}
 	this.init_table = init_table;
-	this.on_before_post = on_before_post;
 }
 
 task.events.events15 = new Events15();
 
+function Events12() { // demo.catalogs.albums 
+
+	function init_table(item, options) {
+		item.selections = [];	
+		options.selections = item.selections;	
+		options.select_all = true;
+	}
+	this.init_table = init_table;
+}
+
+task.events.events12 = new Events12();
+
 function Events19() { // demo.reports.invoice 
 
 	function on_before_print_report(report) {
