@@ -1,16 +1,15 @@
-# -*- coding: utf-8 -*-
-
 import datetime
 import traceback
-import common
 
-FIELD_DEF = FIELD_ID, FIELD_NAME, NAME, FIELD_DATA_TYPE, REQUIRED, LOOKUP_ITEM, MASTER_FIELD, LOOKUP_FIELD, \
-    FIELD_VISIBLE, FIELD_VIEW_INDEX, FIELD_EDIT_VISIBLE, FIELD_EDIT_INDEX, FIELD_READ_ONLY, FIELD_EXPAND, FIELD_WORD_WRAP, \
-    FIELD_SIZE, FIELD_DEFAULT_VALUE, FIELD_DEFAULT, FIELD_CALCULATED, FIELD_EDITABLE, FIELD_ALIGNMENT, FIELD_LOOKUP_VALUES, \
-    FIELD_ENABLE_TYPEAHEAD, FIELD_HELP, FIELD_PLACEHOLDER = range(25)
+import jam.common as common
 
-FILTER_DEF = FILTER_OBJ_NAME, FILTER_NAME, FILTER_FIELD_NAME, FILTER_TYPE, FILTER_DATA_TYPE, FILTER_VISIBLE, \
-    FILTER_HELP, FILTER_PLACEHOLDER = range(8)
+FIELD_DEF = FIELD_ID, FIELD_NAME, NAME, FIELD_DATA_TYPE, REQUIRED, LOOKUP_ITEM, MASTER_FIELD, LOOKUP_FIELD, LOOKUP_FIELD1, \
+    LOOKUP_FIELD2, FIELD_VISIBLE, FIELD_VIEW_INDEX, FIELD_EDIT_VISIBLE, FIELD_EDIT_INDEX, FIELD_READ_ONLY, FIELD_EXPAND, \
+    FIELD_WORD_WRAP, FIELD_SIZE, FIELD_DEFAULT_VALUE, FIELD_DEFAULT, FIELD_CALCULATED, FIELD_EDITABLE, FIELD_ALIGNMENT, \
+    FIELD_LOOKUP_VALUES, FIELD_MULTI_SELECT, FIELD_MULTI_SELECT_ALL, FIELD_ENABLE_TYPEAHEAD, FIELD_HELP, FIELD_PLACEHOLDER = range(29)
+
+FILTER_DEF = FILTER_OBJ_NAME, FILTER_NAME, FILTER_FIELD_NAME, FILTER_TYPE, FILTER_MULTI_SELECT, FILTER_DATA_TYPE, \
+    FILTER_VISIBLE, FILTER_HELP, FILTER_PLACEHOLDER = range(9)
 
 class DatasetException(Exception):
     pass
@@ -28,6 +27,10 @@ class DBField(object):
         self.lookup_item = field_def[LOOKUP_ITEM]
         self.master_field = field_def[MASTER_FIELD]
         self.lookup_field = field_def[LOOKUP_FIELD]
+        self.lookup_item1 = None
+        self.lookup_field1 = field_def[LOOKUP_FIELD1]
+        self.lookup_item2 = None
+        self.lookup_field2 = field_def[LOOKUP_FIELD2]
         self.read_only = field_def[FIELD_READ_ONLY]
         self.view_visible = field_def[FIELD_VISIBLE]
         self.view_index = field_def[FIELD_VIEW_INDEX]
@@ -42,16 +45,22 @@ class DBField(object):
         self.editable = field_def[FIELD_EDITABLE]
         self.alignment = field_def[FIELD_ALIGNMENT]
         self.lookup_values = field_def[FIELD_LOOKUP_VALUES]
+        self.multi_select = field_def[FIELD_MULTI_SELECT]
+        self.multi_select_all = field_def[FIELD_MULTI_SELECT_ALL]
+        self.enable_typeahead = field_def[FIELD_ENABLE_TYPEAHEAD]
+        self.field_help = field_def[FIELD_HELP]
+        self.field_placeholder = field_def[FIELD_PLACEHOLDER]
+
         self.field_type = common.FIELD_TYPE_NAMES[self.data_type]
         self.filter = None
-        self.on_get_field_text_called = None
+        self.on_field_get_text_called = None
 
     def get_row(self):
         if self.owner._dataset:
             return self.owner._dataset[self.owner.rec_no]
         else:
-            print traceback.format_exc()
-            raise Exception, u'An attempt to get a field value in the empty dataset - item: %s, field: %s ' % (self.owner.item_name, self.field_name)
+            traceback.print_exc()
+            raise Exception(self.owner.task.lang['value_in_empty_dataset'] % self.owner.item_name)
 
     def get_data(self):
         row = self.get_row()
@@ -100,18 +109,18 @@ class DBField(object):
                         if self.owner:
                             result = self.owner.task.lang['yes']
                         else:
-                            result = u'True'
+                            result = 'True'
                     else:
                         if self.owner:
                             result = self.owner.task.lang['no']
                         else:
-                            result = u'False'
+                            result = 'False'
                 else:
                     result = str(result)
             else:
                 result = ''
-        except Exception, e:
-            print traceback.format_exc()
+        except Exception as e:
+            traceback.print_exc()
             self.do_on_error(self.type_error() % (''), e)
         return result
 
@@ -137,14 +146,14 @@ class DBField(object):
                         else:
                             self.set_value(False)
                     else:
-                        if len(value) and value[0] in [u'T', u't']:
+                        if len(value) and value[0] in ['T', 't']:
                             self.set_value(True)
                         else:
                             self.set_value(False)
                 else:
                     self.set_value(value)
-            except Exception, e:
-                print traceback.format_exc()
+            except Exception as e:
+                traceback.print_exc()
                 self.do_on_error(self.type_error() % (value), e)
 
     text = property (get_text, set_text)
@@ -175,8 +184,8 @@ class DBField(object):
                 if type(value) in [str, unicode]:
                     value = self.convert_date_time(value)
             return value
-        except Exception, e:
-            print traceback.format_exc()
+        except Exception as e:
+            traceback.print_exc()
             self.do_on_error(self.type_error() % (''), e)
 
     raw_value = property (get_raw_value)
@@ -202,8 +211,8 @@ class DBField(object):
                     else:
                         value = False
             return value
-        except Exception, e:
-            print traceback.format_exc()
+        except Exception as e:
+            traceback.print_exc()
             self.do_on_error(self.type_error() % (''), e)
 
     def _change_lookup_field(self, lookup_value=None):
@@ -222,31 +231,23 @@ class DBField(object):
     def _do_before_changed(self):
         if self.owner:
             if not self.owner.item_state in (common.STATE_INSERT, common.STATE_EDIT):
-                raise DatasetException, u'%s is not in edit or insert mode' % self.owner.item_name
+                raise DatasetException(self.owner.task.lang['not_edit_insert_state'] % self.owner.item_name)
             if self.owner.on_before_field_changed:
                 self.owner.on_before_field_changed(self)
 
     def _check_system_field_value(self, value):
         if self.field_kind == common.ITEM_FIELD:
             if self.field_name == self.owner._primary_key and self.value and self.value != value:
-                raise DatasetException, u'%s: сhanging of the primary field is forbidden' % self.owner.item_name
+                raise DatasetException(self.owner.task.lang['no_primary_field_changing'] % self.owner.item_name)
             if self.field_name == self.owner._deleted_flag and self.value != value:
-                raise DatasetException, u'%s, field - %s: сhanging of the deleted flag field is forbidden' % (self.owner.item_name, self.field_name)
-
-    def _is_filter_value(self, value):
-        if self.field_kind == common.FILTER_FIELD:
-            if self.filter.filter_type in (common.FILTER_IN, common.FILTER_NOT_IN):
-                if not type(value) in [list, tuple]:
-                    error = u'Filter "%s" value must be an array' % self.filter.filter_name
-                    self.do_on_error(error);
-                return True
+                raise DatasetException(self.owner.task.lang['no_deleted_field_changing'] % self.owner.item_name)
 
     def set_value(self, value, lookup_value=None, lookup_item=None):
         self._check_system_field_value(value)
         self.new_value = None
         if not value is None:
             self.new_value = value
-            if not self._is_filter_value(value):
+            if not self.multi_select:
                 if self.data_type == common.BOOLEAN:
                     if bool(value):
                         self.new_value = 1
@@ -260,8 +261,8 @@ class DBField(object):
             self._do_before_changed()
             try:
                 self.set_data(self.new_value)
-            except Exception, e:
-                print traceback.format_exc()
+            except Exception as e:
+                traceback.print_exc()
                 self.do_on_error(self.type_error() % (value), e)
             finally:
                 self.new_value = None
@@ -322,8 +323,8 @@ class DBField(object):
                         elif data_type == common.CURRENCY:
                             result = self.cur_to_str(result)
             result = unicode(result)
-        except Exception, e:
-            print traceback.format_exc()
+        except Exception as e:
+            traceback.print_exc()
         return result
 
     lookup_text = property (get_lookup_text)
@@ -354,15 +355,15 @@ class DBField(object):
             else:
                 result = self.text
         if self.owner and not self.filter:
-            if self.owner.on_get_field_text:
-                if not self.on_get_field_text_called:
-                    self.on_get_field_text_called = True
+            if self.owner.on_field_get_text:
+                if not self.on_field_get_text_called:
+                    self.on_field_get_text_called = True
                     try:
-                        res = self.owner.on_get_field_text(self)
+                        res = self.owner.on_field_get_text(self)
                         if not res is None:
                             result = res
                     finally:
-                        self.on_get_field_text_called = False
+                        self.on_field_get_text_called = False
         return result
 
     display_text = property(get_display_text)
@@ -418,7 +419,7 @@ class DBField(object):
         self.get_value()
         if (self.data_type == common.TEXT) and (self.field_size != 0) and \
             (len(self.text) > self.field_size):
-            print self.text, len(self.text), type(self.text)
+            print(self.text, len(self.text), type(self.text))
             self.do_on_error(self.owner.task.lang['invalid_length'] % self.field_size)
         return True
 
@@ -456,8 +457,8 @@ class DBField(object):
             mess = str(err)
         if mess:
             if self.owner:
-                print u'%s %s - %s: %s' % (self.owner.item_name,
-                    self.owner.task.lang['error'].lower(), self.field_name, mess)
+                print('%s %s - %s: %s' % (self.owner.item_name,
+                    self.owner.task.lang['error'].lower(), self.field_name, mess))
         raise TypeError(mess)
 
     def type_error(self):
@@ -546,6 +547,7 @@ class DBFilter(object):
         self.field_name = filter_def[FILTER_FIELD_NAME]
         self.filter_type = filter_def[FILTER_TYPE]
         self.data_type = filter_def[FILTER_DATA_TYPE]
+        self.multi_select_all = filter_def[FILTER_MULTI_SELECT]
         self.visible = filter_def[FILTER_VISIBLE]
         self.filter_help = filter_def[FILTER_HELP]
         if type(self.field_name) == int:
@@ -554,6 +556,8 @@ class DBFilter(object):
         if self.field_name:
             self.field = FilterField(self, field, self.owner)
             setattr(self, self.field_name, self.field)
+            if self.filter_type in (common.FILTER_IN, common.FILTER_NOT_IN):
+                self.field.multi_select = True;
             if self.filter_type == common.FILTER_RANGE:
                 self.field1 = FilterField(self, field, self.owner)
 
@@ -693,7 +697,7 @@ class ChangeLog(object):
                 else:
                     self.item.record_status = common.RECORD_DELETED
             else:
-                raise Exception, u'%s: change log invalid records state' % self.item.item_name
+                raise Exception('Item %s: change log invalid records state' % self.item.item_name)
             if self.item.master:
                 if self.item.master.record_status == common.RECORD_UNCHANGED:
                     self.item.master.record_status = common.RECORD_DETAILS_MODIFIED
@@ -929,9 +933,7 @@ class AbstractDataSet(object):
         self._filtered = False
         self.expanded = True
         self._open_params = {}
-        self.post_local = False
         self._disabled_count = 0
-        self.open_params = {}
         self._is_delta = False
         self.keep_history = False
         self.parent_read_only = True
@@ -957,7 +959,7 @@ class AbstractDataSet(object):
         self.on_before_field_changed = None
         self.on_filter_value_changed = None
         self.on_field_validate = None
-        self.on_get_field_text = None
+        self.on_field_get_text = None
 
     def __getitem__(self, key):
         if key == 0:
@@ -969,8 +971,8 @@ class AbstractDataSet(object):
         return self
 
     def add_field_def(self, field_ID, field_name, field_caption, data_type, required, lookup_item, lookup_field,
-            view_visible, view_index, edit_visible, edit_index, read_only, expand, word_wrap,
-            field_size, default_value, is_default, calculated, editable, master_field, alignment,
+            lookup_field1, lookup_field2, view_visible, view_index, edit_visible, edit_index, read_only, expand,
+            word_wrap, field_size, default_value, is_default, calculated, editable, master_field, alignment,
             lookup_values, enable_typeahead, field_help, field_placeholder):
         field_def = [None for i in range(len(FIELD_DEF))]
         field_def[FIELD_ID] = field_ID
@@ -981,6 +983,8 @@ class AbstractDataSet(object):
         field_def[LOOKUP_ITEM] = lookup_item
         field_def[MASTER_FIELD] = master_field
         field_def[LOOKUP_FIELD] = lookup_field
+        field_def[LOOKUP_FIELD1] = lookup_field1
+        field_def[LOOKUP_FIELD2] = lookup_field2
         field_def[FIELD_READ_ONLY] = read_only
         field_def[FIELD_VISIBLE] = view_visible
         field_def[FIELD_VIEW_INDEX] = view_index
@@ -1001,13 +1005,14 @@ class AbstractDataSet(object):
         self.field_defs.append(field_def)
         return field_def
 
-    def add_filter_def(self, filter_name, filter_caption, field_name, filter_type, \
-        data_type, visible, filter_help, filter_placeholder):
+    def add_filter_def(self, filter_name, filter_caption, field_name, filter_type,
+            multi_select_all, data_type, visible, filter_help, filter_placeholder):
         filter_def = [None for i in range(len(FILTER_DEF))]
         filter_def[FILTER_OBJ_NAME] = filter_name
         filter_def[FILTER_NAME] = filter_caption
         filter_def[FILTER_FIELD_NAME] = field_name
         filter_def[FILTER_TYPE] = filter_type
+        filter_def[FILTER_MULTI_SELECT] = multi_select_all
         filter_def[FILTER_DATA_TYPE] = data_type
         filter_def[FILTER_VISIBLE] = visible
         filter_def[FILTER_HELP] = filter_help
@@ -1091,7 +1096,21 @@ class AbstractDataSet(object):
             if field.master_field and type(field.master_field) == int:
                 field.master_field = self._field_by_ID(field.master_field)
             if field.lookup_field and type(field.lookup_field) == int:
-                field.lookup_field = field.lookup_item._field_by_ID(field.lookup_field).field_name
+                lookup_field = field.lookup_item._field_by_ID(field.lookup_field)
+                field.lookup_field = lookup_field.field_name
+                if lookup_field.lookup_item and field.lookup_field1:
+                    field.lookup_item1 = lookup_field.lookup_item
+                    if type(field.lookup_item1) == int:
+                        field.lookup_item1 = self.task.item_by_ID(field.lookup_item1)
+                    if type(field.lookup_field1) == int:
+                        lookup_field1 = field.lookup_item1._field_by_ID(field.lookup_field1)
+                        field.lookup_field1 = lookup_field1.field_name
+                    if lookup_field1.lookup_item and field.lookup_field2:
+                        field.lookup_item2 = lookup_field1.lookup_item
+                        if type(field.lookup_item2) == int:
+                            field.lookup_item2 = self.task.item_by_ID(field.lookup_item2)
+                        if type(field.lookup_field2) == int:
+                            field.lookup_field2 = field.lookup_item2._field_by_ID(field.lookup_field2).field_name
             if field.lookup_values and type(field.lookup_values) == int:
                 try:
                     field.lookup_values = self.task.lookup_lists[field.lookup_values]
@@ -1430,7 +1449,7 @@ class AbstractDataSet(object):
 
     def get_where_list(self, field_dict):
         result = []
-        for field_arg in field_dict.keys():
+        for field_arg in field_dict.iterkeys():
             field_name = field_arg
             pos = field_name.find('__')
             if pos > -1:
@@ -1448,7 +1467,7 @@ class AbstractDataSet(object):
                 raise RuntimeError('%s: set_where method arument error %s: ' % (self.item_name, field_arg))
             value = field_dict[field_arg]
             if not value is None:
-                result.append((field_name, filter_type, value))
+                result.append([field_name, filter_type, value])
         return result
 
     def set_fields(self, *fields):
@@ -1487,7 +1506,7 @@ class AbstractDataSet(object):
                 if field:
                     self.fields.append(field)
                 else:
-                    raise Exception, '%s - _do_before_open method error: there is no field with field_name: %s' % (self.item_name, field_name)
+                    raise Exception('%s - _do_before_open method error: there is no field with field_name: %s' % (self.item_name, field_name))
         else:
             self.fields = list(self._fields)
         for field in self.fields:
@@ -1617,16 +1636,15 @@ class AbstractDataSet(object):
 
     def append(self):
         if not self.active:
-            raise DatasetException(u"Can't insert record in %s: %s is not active" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['append_not_active'] % self.item_name)
         if self.item_state != common.STATE_BROWSE:
-            raise DatasetException(u"Can't insert record in %s: %s is not in browse state" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['append_not_browse'] % self.item_name)
         self._do_before_append()
         self._do_before_scroll()
         self._old_row = self.rec_no
         self.item_state = common.STATE_INSERT
         self._dataset.append(self.new_record())
         self._cur_row = len(self._dataset) - 1
-#        self._modified = False
         self.record_status = common.RECORD_INSERTED
         self.update_controls(common.UPDATE_APPEND)
         self._do_after_scroll()
@@ -1634,9 +1652,9 @@ class AbstractDataSet(object):
 
     def insert(self):
         if not self.active:
-            raise DatasetException(u"Can't insert record in %s: %s is not active" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['insert_not_active'] % self.item_name)
         if self.item_state != common.STATE_BROWSE:
-            raise DatasetException(u"Can't insert record in %s: %s is not in browse state" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['insert_not_browse'] % self.item_name)
         self._do_before_append()
         self._do_before_scroll()
         self._old_row = self.rec_no
@@ -1683,11 +1701,11 @@ class AbstractDataSet(object):
 
     def edit(self):
         if not self.active:
-            raise DatasetException(u"Can't edit record in %s: %s is not active" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['edit_not_active'] % self.item_name)
         if self.item_state != common.STATE_BROWSE:
-            raise DatasetException(u"Can't edit record in %s: %s is not in browse state" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['edit_not_browse'] % self.item_name)
         if self.record_count() == 0:
-            raise DatasetException(u"Can't edit record in %s: %s' record list is empty" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['edit_no_records'] % self.item_name)
         self._do_before_edit()
         self.change_log.log_change()
         self._buffer = self.change_log.store_record_log()
@@ -1707,9 +1725,9 @@ class AbstractDataSet(object):
 
     def delete(self):
         if not self.active:
-            raise DatasetException(u"Can't delete record in %s: %s is not active" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['delete_not_active'] % self.item_name)
         if self.record_count() == 0:
-            raise DatasetException(u"Can't delete record in %s: %s' record list is empty" % (self.item_name, self.item_name))
+            raise DatasetException(self.task.lang['delete_no_records'] % self.item_name)
         self.item_state = common.STATE_DELETE
         try:
             self._do_before_delete()
@@ -1746,7 +1764,7 @@ class AbstractDataSet(object):
             self.update_controls(common.UPDATE_DELETE)
             del self._dataset[self.rec_no]
         else:
-            raise Exception, '%s cancel error: invalid item state' % self.item_name
+            raise Exception(self.task.lang['cancel_invalid_state'] % self.item_name)
         prev_state = self.item_state
         self.item_state = common.STATE_BROWSE
         if prev_state in [common.STATE_INSERT]:
@@ -1769,7 +1787,7 @@ class AbstractDataSet(object):
 
     def post(self):
         if not self.is_changing():
-            raise DatasetException, u'%s: dataset is not in edit or insert mode' % self.item_name
+            raise DatasetException(self.task.lang['not_edit_insert_state'] % self.item_name)
         self.check_record_valid()
         self._do_before_post()
         if self.master:
@@ -1859,19 +1877,25 @@ class AbstractDataSet(object):
 
     def set_filters(self, **filters):
         self.clear_filters()
-        for filter_name in filters.keys():
+        for filter_name in filters.iterkeys():
             try:
                 filter = self.filter_by_name(filter_name)
                 filter.value = filters[filter_name]
-            except Exception, e:
+            except Exception as e:
                 raise RuntimeError('%s: set_filters method arument error %s=%s: %s' % (self.item_name, filter_name, filters[filter_name], e))
 
-    def find_default_field(self):
-        for field in self.fields:
-            if field.is_default:
-                return field
+    def get_default_field(self):
+        try:
+            return self._default_field
+        except:
+            self._default_field = None
+            for field in self.fields:
+                if field.is_default:
+                    self._default_field = field
+                    break
+            return self._default_field
 
-    default_field = property (find_default_field)
+    default_field = property (get_default_field)
 
     def round(self, value, dec):
         return round(value, dec)
@@ -1898,10 +1922,10 @@ class MasterDataSet(AbstractDataSet):
 
         return result
 
-    def do_apply(self, params):
+    def do_apply(self, params, safe):
         pass
 
-    def apply(self, params=None):
+    def apply(self, params=None, safe=False):
         result = None
         if self.is_changing():
             self.post()
@@ -1909,7 +1933,7 @@ class MasterDataSet(AbstractDataSet):
             result = self.on_before_apply(self)
             if result:
                 params = result
-        self.do_apply(params)
+        self.do_apply(params, safe)
         if self.on_after_apply:
             self.on_after_apply(self)
 
@@ -1977,7 +2001,11 @@ class MasterDetailDataset(MasterDataSet):
 
     def open(self, expanded=None, fields=None, where=None, order_by=None,
         open_empty=False, params=None, offset=None, limit=None, funcs=None,
-        group_by=None):
+        group_by=None, safe=False):
+        if safe and self.session and not self.master:
+            priv = self.session.find_privileges(self)
+            if priv and not priv['can_view']:
+                raise Exception(self.task.lang['cant_view'] % self.item_caption)
         if expanded is None:
             expanded = self.expanded
         else:
@@ -2037,12 +2065,13 @@ class MasterDetailDataset(MasterDataSet):
             if not args in ['self', 'ids', 'params', 'slice_ids']:
                 params[args] = value
         if type(ids) == dict:
-            id_field_name = ids.keys()[0]
+            keys = list(ids.iterkeys())
+            id_field_name = keys[0]
             ids = ids[id_field_name]
         elif type(ids) == list:
             id_field_name = 'id'
         else:
-            raise Exception, u'Invalid ids parameter in open_for_ids method of item "%s"' % self.item_name
+            raise Exception('Item %s: invalid ids parameter in open__in method' % self.item_name)
         if params['where'] is None:
             params['where'] = {}
         on_after_open = self.on_after_open
@@ -2066,22 +2095,22 @@ class MasterDetailDataset(MasterDataSet):
 
     def insert(self):
         if self.master and not self.master.is_changing():
-            raise DatasetException, u"%s: can't insert record - master item is not in edit or insert mode" % self.owner.item_name
+            raise DatasetException(self.task.lang['insert_master_not_changing'] % self.item_name)
         super(MasterDetailDataset, self).insert()
 
     def append(self):
         if self.master and not self.master.is_changing():
-            raise DatasetException, u"%s: can't append record - master item is not in edit or insert mode" % self.owner.item_name
+            raise DatasetException(self.task.lang['append_master_not_changing'] % self.item_name)
         super(MasterDetailDataset, self).append()
 
     def edit(self):
         if self.master and not self.master.is_changing():
-            raise DatasetException, u"%s: can't edit record - master item is not in edit or insert mode" % self.owner.item_name
+            raise DatasetException(self.task.lang['edit_master_not_changing'] % self.item_name)
         super(MasterDetailDataset, self).edit()
 
     def delete(self):
         if self.master and not self.master.is_changing():
-            raise DatasetException, u"%s: can't delete record - master item is not in edit or insert mode" % self.owner.item_name
+            raise DatasetException(self.task.lang['delete_master_not_changing'] % self.item_name)
         super(MasterDetailDataset, self).delete()
 
     def _set_modified(self, value):
