@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import with_statement
 import sys, os
 
 import Queue
@@ -9,17 +6,16 @@ import threading
 import zipfile
 from xml.dom.minidom import parseString
 from xml.sax.saxutils import escape
-import uuid
 import datetime, time
 import traceback
 import inspect
 import json
 
-import common, db.db_modules as db_modules
-from items import *
-from dataset import *
-from sql import *
-import adm_server
+import jam.common as common
+import jam.db.db_modules as db_modules
+from jam.items import *
+from jam.dataset import *
+from jam.sql import *
 
 class ServerDataset(Dataset, SQL):
     def __init__(self, table_name='', soft_delete=True):
@@ -31,8 +27,7 @@ class ServerDataset(Dataset, SQL):
         self.on_open = None
         self.on_apply = None
         self.on_count = None
-        self.on_get_field_text = None
-        self.deleted_field_name = None
+        self.on_field_get_text = None
         self.soft_delete = soft_delete
         self.virtual_table = False
 
@@ -58,20 +53,23 @@ class ServerDataset(Dataset, SQL):
     def add_field(self, field_id, field_name, field_caption, data_type, required = False,
         item = None, object_field = None, visible = True, index=0, edit_visible = True, edit_index = 0, read_only = False,
         expand = False, word_wrap = False, size = 0, default_value=None, default = False, calculated = False, editable = False,
-        master_field = None, alignment=None, lookup_values=None, enable_typeahead=False, field_help=None, field_placeholder=None):
+        master_field = None, alignment=None, lookup_values=None, enable_typeahead=False, field_help=None,
+        field_placeholder=None, lookup_field1=None, lookup_field2=None):
 
-        field_def = self.add_field_def(field_id, field_name, field_caption, data_type, required, item, object_field, visible,
-            index, edit_visible, edit_index, read_only, expand, word_wrap, size, default_value, default, calculated, editable,
-            master_field, alignment, lookup_values, enable_typeahead, field_help, field_placeholder)
+        field_def = self.add_field_def(field_id, field_name, field_caption, data_type, required, item, object_field,
+            lookup_field1, lookup_field2, visible, index, edit_visible, edit_index, read_only, expand, word_wrap, size,
+            default_value, default, calculated, editable, master_field, alignment, lookup_values, enable_typeahead,
+            field_help, field_placeholder)
         field = DBField(self, field_def)
         self._fields.append(field)
         return field
 
     def add_filter(self, name, caption, field_name, filter_type = common.FILTER_EQ,
-        data_type = None, visible = True, filter_help=None, filter_placeholder=None):
+        multi_select_all=None, data_type = None, visible = True, filter_help=None,
+        filter_placeholder=None):
 
         filter_def = self.add_filter_def(name, caption, field_name, filter_type,
-            data_type, visible, filter_help, filter_placeholder)
+            multi_select_all, data_type, visible, filter_help, filter_placeholder)
         fltr = DBFilter(self, filter_def)
         self.filters.append(fltr)
         return fltr
@@ -79,17 +77,14 @@ class ServerDataset(Dataset, SQL):
     def do_internal_open(self, params):
         return self.select_records(params)
 
-    def do_apply(self, params=None):
+    def do_apply(self, params=None, safe=False):
         if not self.master and self.log_changes:
-            if self.item_state != common.STATE_BROWSE:
-                raise Exception, u'Item: %s is not in browse state. Apply requires browse state.'
             changes = {}
             self.change_log.get_changes(changes)
             if changes['data']:
-                data, error = self.apply_changes((changes, params),
-                    {'can_view': True, 'can_create': True, 'can_edit': True, 'can_delete': True})
+                data, error = self.apply_changes((changes, params), safe)
                 if error:
-                    raise Exception, error
+                    raise Exception(error)
                 else:
                     self.change_log.update(data)
 
@@ -120,46 +115,58 @@ class ServerDataset(Dataset, SQL):
             self._order_by.append([fld.ID, desc])
         return self
 
-    def get_record_count(self, params, user_info=None, enviroment=None):
-        result = 0;
-        if self.on_count:
-            result, error_mes = self.on_count(self, params, user_info, enviroment)
-        else:
-            error_mes = ''
-            result = 0
+    def get_record_count(self, params, safe=False):
+        if safe and self.session and not self.master:
+            priv = self.session.find_privileges(self)
+            if priv and not priv['can_view']:
+                raise Exception(self.task.lang['cant_view'] % self.item_caption)
+        result = None
+        if self.task.on_count:
+            result = self.task.on_count(self, params)
+        if result is None and self.on_count:
+            result = self.on_count(self, params)
+        elif result is None:
+            error_mess = ''
             sql = self.get_record_count_query(params)
             rows = self.task.execute_select(sql)
-            result = rows[0][0]
-        return result, error_mes
+            count = rows[0][0]
+            result = count, error_mess
+        return result
 
-    def select_records(self, params, user_info=None, enviroment=None):
-        if self.on_open:
-            rows, error_mes = self.on_open(self, params, user_info, enviroment)
-        else:
-            sql = self.get_select_statement(params)
+    def select_records(self, params, safe=False):
+        if safe and self.session and not self.master:
+            priv = self.session.find_privileges(self)
+            if priv and not priv['can_view']:
+                raise Exception(self.task.lang['cant_view'] % self.item_caption)
+        result = None
+        if self.task.on_open:
+            result = self.task.on_open(self, params)
+        if result is None and self.on_open:
+            result = self.on_open(self, params)
+        elif result is None:
             error_mes = ''
-            rows = []
+            sql = self.get_select_statement(params)
             rows = self.task.execute_select(sql)
-        return rows, error_mes
+            result = rows, error_mes
+        return result
 
-    def apply_delta(self, delta, privileges=None, user_info=None):
-        sql = delta.apply_sql(user_info, privileges)
+    def apply_delta(self, delta, safe=False):
+        sql = delta.apply_sql(safe)
         return self.task.execute(sql)
 
-    def apply_changes(self, data, privileges, user_info=None, enviroment=None):
-        if common.DEMO:
-            raise Exception, u'Demo version - changes are not allowed.'
-        error = None
+    def apply_changes(self, data, safe):
         result = None
         changes, params = data
         if not params:
             params = {}
         delta = self.delta(changes)
-        if self.on_apply:
-            result, error = self.on_apply(self, delta, params, privileges, user_info, enviroment)
-        else:
-            result, error = self.apply_delta(delta, privileges, user_info)
-        return result, error
+        if self.task.on_apply:
+            result = self.task.on_apply(self, delta, params)
+        if result is None and self.on_apply:
+            result = self.on_apply(self, delta, params)
+        elif result is None:
+            result = self.apply_delta(delta, safe)
+        return result
 
     def update_deleted(self):
         if self._is_delta and len(self.details):
@@ -314,21 +321,23 @@ class Report(AbstrReport):
         self.on_before_field_changed = None
         self.on_filter_value_changed = None
         self.on_field_validate = None
-        self.on_get_field_text = None
+        self.on_field_get_text = None
 
 
     def add_param(self, caption='', name='', data_type=common.INTEGER,
-        obj=None, obj_field=None, required=True, visible=True, alignment=None,
-        enable_typeahead=None, lookup_values=None, param_help=None, param_placeholder=None):
+            obj=None, obj_field=None, required=True, visible=True, alignment=None,
+            multi_select=None, multi_select_all=None, enable_typeahead=None, lookup_values=None,
+            param_help=None, param_placeholder=None):
         param_def = self.add_param_def(caption, name, data_type, obj,
-            obj_field, required, visible, alignment, enable_typeahead,
-            lookup_values, param_help, param_placeholder)
+            obj_field, required, visible, alignment, multi_select, multi_select_all,
+            enable_typeahead, lookup_values, param_help, param_placeholder)
         param = Param(self, param_def)
         self.params.append(param)
 
     def add_param_def(self, param_caption='', param_name='', data_type=common.INTEGER,
-        lookup_item=None, lookup_field=None, required=True, visible=True,
-        alignment=0, enable_typeahead=False, lookup_values=None, param_help=None,
+            lookup_item=None, lookup_field=None, required=True, visible=True,
+            alignment=0, multi_select=False, multi_select_all=False, enable_typeahead=False,
+            lookup_values=None, param_help=None,
         param_placeholder=None):
         param_def = [None for i in range(len(FIELD_DEF))]
         param_def[FIELD_NAME] = param_name
@@ -339,6 +348,8 @@ class Report(AbstrReport):
         param_def[LOOKUP_FIELD] = lookup_field
         param_def[FIELD_EDIT_VISIBLE] = visible
         param_def[FIELD_ALIGNMENT] = alignment
+        param_def[FIELD_MULTI_SELECT] = multi_select
+        param_def[FIELD_MULTI_SELECT_ALL] = multi_select_all
         param_def[FIELD_ENABLE_TYPEAHEAD] = enable_typeahead
         param_def[FIELD_LOOKUP_VALUES] = lookup_values
         param_def[FIELD_HELP] = param_help
@@ -370,7 +381,11 @@ class Report(AbstrReport):
         result.prepare_params()
         return  result
 
-    def print_report(self, param_values, url, ext=None):
+    def print_report(self, param_values, url, ext=None, safe=False):
+        if safe and self.session:
+            priv = self.session.find_privileges(self)
+            if priv and not priv['can_view']:
+                raise Exception(self.task.lang['cant_view'] % self.item_caption)
         if not self.template_content:
             self.parse_template()
         copy_report = self.copy()
@@ -380,10 +395,6 @@ class Report(AbstrReport):
         if not ext:
             ext = 'ods'
         file_name = self.item_name + '_' + datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S.%f') + '.' + ext
-        #~ if os.name == "nt":
-            #~ file_name = self.item_name + '_' + datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S.%f') + '.' + ext
-        #~ else:
-            #~ file_name = self.item_caption + '_' + datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S.%f') + '.' + ext
         file_name = escape(file_name, {':': '-', '/': '_', '\\': '_'})
         return os.path.abspath(os.path.join(self.task.work_dir, 'static', 'reports', file_name))
 
@@ -515,7 +526,7 @@ class Report(AbstrReport):
                     start = end + 2
                 columns_end = start
                 header = data[0:columns_start]
-                assert len(band_tags) > 0, u'No bands in report template'
+                assert len(band_tags) > 0, 'No bands in the report template'
                 positions = []
                 start = 0
                 for tag in band_tags:
@@ -601,7 +612,7 @@ class Report(AbstrReport):
         text = self.template_content['bands'][band]
         if dic:
             d = dic.copy()
-            for key, value in d.items():
+            for key, value in d.iteritems():
                 if type(value) in (str, unicode):
                     d[key] = escape(value)
             cell_start = 0
@@ -635,8 +646,9 @@ class Report(AbstrReport):
                                         if type(value) == float:
                                             val = val.replace('.', common.DECIMAL_POINT)
                                     else:
-                                        if not key in d.keys():
-                                            print 'Report: "%s" band: "%s" key "%s" not found in the dictionary' % (self.item_name, band, key)
+                                        if not key in d.iterkeys():
+                                            print('Report: "%s" band: "%s" key "%s" not found in the dictionary' % \
+                                                (self.item_name, band, key))
                                     if  isinstance(cell_text, unicode):
                                         cell_text = cell_text.encode('utf8')
                                     if  isinstance(val, unicode):
@@ -687,200 +699,18 @@ class Report(AbstrReport):
     def _set_modified(self, value):
         pass
 
-#~ log = {}
-#~ log_counter = 0
-#~ log_save_after = 0
-#~ log_pid = os.getpid()
-#~
-#~ def find_where_fields(command):
-    #~ result = ''
-    #~ where_pos = command.find('WHERE')
-    #~ where = None
-    #~ if where_pos >= 0:
-        #~ a = []
-        #~ where = command[where_pos + 5:]
-        #~ a.append(where.find(' GROUP BY'))
-        #~ a.append(where.find(' ORDER BY'))
-        #~ a.append(where.find(' HAVING'))
-        #~ pos = -1
-        #~ b = []
-        #~ for p in a:
-            #~ if p >= 0:
-                #~ b.append(p)
-        #~ if len(b):
-            #~ pos = min(b)
-        #~ if pos >= 0:
-            #~ where = where[:pos + 1]
-        #~ if where:
-            #~ parts = where.split('AND')
-            #~ fields = []
-            #~ for p in parts:
-                #~ p = p.strip()
-                #~ p = p.strip('(')
-                #~ w = ''
-                #~ for ch in p:
-                    #~ if ch in [' ', '>', '<', '=']:
-                        #~ break
-                    #~ else:
-                        #~ w += ch
-                #~ w = w.replace('"', '')
-                #~ pos = w.find('.')
-                #~ if pos >= 0:
-                    #~ w = w[pos + 1:]
-                #~ fields.append(w)
-            #~ result = ' '.join(fields)
-    #~ return result
-#~
-#~ def find_table_name(command):
-    #~ result = ''
-    #~ s = ''
-    #~ if command[:12] ==   'SELECT NEXT ':
-        #~ result = ''
-    #~ elif command[:12] == 'INSERT INTO ':
-        #~ result = ''
-    #~ elif command[:6] == 'UPDATE':
-        #~ s = command[7:]
-        #~ s = s.lstrip('"')
-    #~ else:
-        #~ pos = command.find(' FROM ')
-        #~ s = command[pos+5:pos+105]
-        #~ s = s.lstrip(' ')
-        #~ s = s.lstrip('(')
-        #~ s = s.lstrip('"')
-    #~ if s:
-        #~ w = ''
-        #~ for ch in s:
-            #~ if ch in [' ', '"']:
-                #~ break
-            #~ else:
-                #~ w += ch
-        #~ result = w
-    #~ return result
-#~
-#~ def save_log(log):
-#~
-    #~ def get_reqs_key(item):
-        #~ key, (dur, freq) = item
-        #~ return dur / 1000000.0 / freq
-#~
-    #~ def get_tables_key(item):
-        #~ key, (dur, freq, reqs) = item
-        #~ return dur / 1000000.0 / freq
-#~
-    #~ text = ''
-#~
-    #~ tlist = sorted(log.iteritems(), key=get_tables_key, reverse=True)
-    #~ for table, (dur, freq, reqs) in tlist:
-        #~ sec = dur / 1000000.0
-        #~ per_request = sec / freq
-        #~ text += '%-20s %f %d %f\n' % (table, per_request, freq, sec)
-#~
-    #~ text += '\n'
-#~
-    #~ tables = sorted(log.keys())
-    #~ for table in tables:
-        #~ dur, freq, reqs = log[table]
-        #~ sec = dur / 1000000.0
-        #~ per_request = sec / freq
-        #~ text += '\n%-20s %f %d %f\n' % (table, per_request, freq, sec)
-        #~ rlist = sorted(reqs.iteritems(), key=get_reqs_key, reverse=True)
-        #~ for fields, (dur, freq) in rlist:
-            #~ sec = dur / 1000000.0
-            #~ per_request = sec / freq
-            #~ text += '      %f %d %f %s\n' % (per_request, freq, sec, fields)
-    #~ with open('log.info', 'w') as f:
-        #~ f.write(text.encode('utf-8'))
-#~
-#~
-#~ def log_request(command, duration):
-    #~ global log_counter
-    #~ global log_pid
-    #~ global log
-    #~ global log_save_after
-#~
-    #~ if log_pid == os.getpid():
-        #~ command = command.upper().strip()
-        #~ table_name = find_table_name(command)
-        #~ if table_name:
-            #~ where_fields = find_where_fields(command)
-#~
-            #~ msec = duration.microseconds
-#~
-            #~ table = log.get(table_name)
-            #~ if not table:
-                #~ table = [0, 0, {}]
-                #~ log[table_name] = table
-            #~ table[0] += msec
-            #~ table[1] += 1
-            #~ reqs = table[2]
-#~
-            #~ req = reqs.get(where_fields)
-            #~ if not req:
-                #~ req = [0, 0]
-                #~ reqs[where_fields] = req
-            #~ req[0] += msec
-            #~ req[1] += 1
-#~
-        #~ log_counter += 1
-        #~ if log_counter % log_save_after == 0:
-            #~ save_log(log)
-#~
-
 def execute_sql(db_module, db_database, db_user, db_password,
     db_host, db_port, db_encoding, connection, command,
     params=None, call_proc=False, commit=True, select=False, ddl=False):
 
-    #~ def execute_command(cursor, command, params=None):
-        #~ global log_save_after
-        #~ print_command = False
-        #~ log_command = False
-        #~ try:
-            #~ if print_command:
-                #~ print ''
-                #~ print command
-                #~ print params
-#~
-            #~ now = datetime.datetime.now()
-#~
-            #~ result = None
-            #~ if params:
-                #~ cursor.execute(command, params)
-            #~ else:
-                #~ cursor.execute(command)
-            #~ if result_set == 'ONE':
-                #~ result = cursor.fetchone()
-            #~ elif result_set == 'ALL':
-                #~ result = cursor.fetchall()
-#~
-            #~ if log_save_after:
-                #~ log_request(command, datetime.datetime.now() - now)
-#~
-            #~ if log_command and command.upper().find('SELECT') == -1:
-                #~ try:
-                    #~ with open("sql_log.txt", "a") as f:
-                        #~ f.write('\n')
-                        #~ f.write(command + '\n')
-                        #~ f.write(json.dumps(params, default=common.json_defaul_handler) + '\n')
-                #~ except:
-                    #~ pass
-#~
-            #~ return result
-        #~ except Exception, x:
-            #~ error = '\nError: %s\n command: %s\n params: %s' % (str(x), command, params)
-            #~ print error
-            #~ if not ddl:
-                #~ raise
-            #~ elif db_module.DDL_ROLLBACK:
-                #~ raise
-
     def execute_command(cursor, command, params=None):
         print_command = ddl
         try:
             if print_command:
-                print ''
-                print command
+                print('')
+                print(command)
                 if params:
-                    print params
+                    print(params)
                     messages.append('<p>' + command + '<br>' + \
                         json.dumps(params, default=common.json_defaul_handler) + '</p>')
                 else:
@@ -899,7 +729,7 @@ def execute_sql(db_module, db_database, db_user, db_password,
             return result
         except Exception, x:
             error = '\nError: %s\n command: %s\n params: %s' % (str(x), command, params)
-            print error
+            print(error)
             if ddl:
                 arr = str(x).split('\\n')
                 error = '<br>'.join(arr)
@@ -977,7 +807,7 @@ def execute_sql(db_module, db_database, db_user, db_password,
                     elif isinstance(com, tuple):
                         res = execute_command(cursor, com[0], com[1])
                     else:
-                        raise Exception, 'server_classes execute_list: invalid argument - command: %s' % command
+                        raise Exception('server_classes execute_list: invalid argument - command: %s' % command)
             return res
 
     def execute(connection):
@@ -991,7 +821,7 @@ def execute_sql(db_module, db_database, db_user, db_password,
                     cursor.callproc(command, params)
                     result = cursor.fetchone()
                 except Exception, x:
-                    print '\nError: %s in command: %s' % (str(x), command)
+                    print('\nError: %s in command: %s' % (str(x), command))
                     raise
             else:
                 if isinstance(command, str) or isinstance(command, unicode):
@@ -1016,7 +846,7 @@ def execute_sql(db_module, db_database, db_user, db_password,
                 error = str(x)
                 if not error:
                     error = 'SQL execution error'
-                print traceback.format_exc()
+                traceback.print_exc()
             finally:
                 connection = None
         finally:
@@ -1035,7 +865,7 @@ def execute_sql(db_module, db_database, db_user, db_password,
         try:
             connection = db_module.connect(db_database, db_user, db_password, db_host, db_port, db_encoding)
         except Exception, x:
-             print str(x)
+             print(str(x))
              return  None, (None, str(x))
     return execute(connection)
 
@@ -1137,6 +967,9 @@ class AbstractServerTask(AbstrTask):
         self.db_encoding = encoding
         self.db_module = db_modules.get_db_module(self.db_type)
         self.on_request = None
+        self.on_open = None
+        self.on_apply = None
+        self.on_count = None
         self.work_dir = os.getcwd()
         self.con_pool_size = 0
         self.mod_count = 0
@@ -1228,7 +1061,7 @@ class AbstractServerTask(AbstrTask):
     def execute_select(self, command, params=None):
         result, error = self.execute(command, params, commit=False, select=True)
         if error:
-            raise Exception, error
+            raise Exception(error)
         else:
             return result
 
@@ -1249,8 +1082,8 @@ class AbstractServerTask(AbstrTask):
         if code:
             try:
                 code = code.encode('utf-8')
-            except Exception, e:
-                print e
+            except Exception as e:
+                print(e)
             comp_code = compile(code, item.module_name, "exec")
             exec comp_code in item_module.__dict__
 
@@ -1287,8 +1120,8 @@ class AbstractServerTask(AbstrTask):
                     stderr=STDOUT,stdout = PIPE)#, shell=True)
                 out, err = convertion.communicate()
                 converted = True
-            except Exception, e:
-                print e
+            except Exception as e:
+                print(e)
         return converted
 
     def copy_database(self, dbtype, database=None, user=None, password=None,
@@ -1332,10 +1165,10 @@ class AbstractServerTask(AbstrTask):
                                         item.post()
                                     item.apply()
                                 else:
-                                    raise Exception, error
+                                    raise Exception(error)
                                 records = len(result)
                                 loaded += records
-                                print 'coping table %s: %d%%' % (item.item_name, int(loaded * 100 / record_count))
+                                print('coping table %s: %d%%' % (item.item_name, int(loaded * 100 / record_count)))
                                 if records == 0 or records < limit:
                                     break
                             if self.db_module.restart_sequence_sql:
@@ -1356,13 +1189,14 @@ class Task(AbstractServerTask):
             host, port, encoding, con_pool_size, mp_pool, persist_con)
         self.on_created = None
         self.on_login = None
-        self.on_get_user_info = None
         self.on_logout = None
         self.on_ext_request = None
         self.init_dict = {}
-        for key, value in self.__dict__.items():
+        self.roles = None
+        for key, value in self.__dict__.iteritems():
             self.init_dict[key] = value
 
+
 class AdminTask(AbstractServerTask):
     def __init__(self, app, name, caption, js_filename,
         db_type, db_database = '', db_user = '', db_password = '',
@@ -1371,76 +1205,20 @@ class AdminTask(AbstractServerTask):
             db_type, db_database, db_user, db_password, host, port, encoding, 2)
         filepath, filename = os.path.split(__file__)
         self.cur_path = filepath
-        self.users = {}
-        self.roles = None
         self.edited_docs = []
 
     def create_task(self):
-        return adm_server.create_task(self.app)
+        from jam.adm_server import create_task
+        return create_task(self.app)
 
     def reload_task(self):
-        adm_server.reload_task(self)
+        from jam.adm_server import reload_task
+        reload_task(self)
 
     def update_events_code(self):
-        adm_server.update_events_code(self)
+        from jam.adm_server import update_events_code
+        update_events_code(self)
 
-    def login(self, log, psw_hash, is_admin, env):
-        privileges = None
-        if not is_admin and self.app.task and self.app.task.on_login:
-            user_info = self.app.task.on_login(self.app.task, log, psw_hash, env)
-        else:
-            user_info = adm_server.login(self, log, psw_hash, is_admin)
-        user_uuid = None
-        if user_info:
-            for key in self.users.iterkeys():
-                if self.users[key]["user_id"] == user_info["user_id"] and \
-                    self.users[key]["admin"] == user_info["admin"]:
-                    if user_info["admin"]:
-                        adm_server.clear_docs_on_logout(self, user_info)
-                    del self.users[key]
-                    break
-            user_uuid = str(uuid.uuid4())
-            self.users[user_uuid] = user_info
-        return user_uuid, ''
-
-    def logout(self, user_uuid, admin, env):
-        try:
-            user_info = self.users.get(user_uuid)
-            adm_server.clear_docs_on_logout(self, user_info)
-            if user_info:
-                if not admin and self.app.task and self.app.task.on_logout:
-                    self.app.task.on_logout(self.app.task, user_info, env)
-                del self.users[user_uuid]
-        except:
-            pass
-        return None, ''
-
-    def get_user_info(self, user_uuid, admin, env):
-        user_info = self.users.get(user_uuid)
-        if user_info:
-            if not admin or (admin and user_info['admin']):
-                return user_info
-
-    def get_privileges(self, role_id):
-        if self.roles is None:
-            self.roles = adm_server.get_roles(self)
-        return self.roles[role_id]
-
-    def find_privileges(self, user_info, item):
-        if not self.safe_mode or item.master or (item.task == self) or (item == item.task):
-            return {'can_view': True, 'can_create': True, 'can_edit': True, 'can_delete': True}
-        else:
-            try:
-                priv_dic = self.get_privileges(user_info['role_id'])[item.ID]
-            except:
-                priv_dic = None
-            if priv_dic:
-                return priv_dic
-            else:
-                return {'can_view': False, 'can_create': False, 'can_edit': False, 'can_delete': False}
-
-    def has_privilege(self, user_info, item, priv_name):
-        return self.find_privileges(user_info, item)[priv_name]
 
 class Group(AbstrGroup):
     def __init__(self, owner, name, caption, view_template=None, js_filename=None, visible=True, item_type_id=0):
@@ -1504,7 +1282,7 @@ class Detail(AbstrDetail, ServerDataset):
                 result = ' WHERE ' + clause
             return db_module.set_case(result)
         else:
-            raise Exception, 'Invalid request parameter'
+            raise Exception('Invalid request parameter')
 
     def get_filters(self):
         return self.prototype.filters
