@@ -1,12 +1,10 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import with_statement
 import sys
 import json
 import traceback
 
-import common, db.db_modules as db_modules
-from dataset import *
+import jam.common as common
+import jam.db.db_modules as db_modules
+from jam.dataset import *
 
 class SQL(object):
 
@@ -75,32 +73,36 @@ class SQL(object):
                 self._primary_key_field.value)
         return db_module.set_case(sql)
 
-    def apply_sql(self, user_info=None, priv=None, db_module=None):
+    def apply_sql(self, safe=False, db_module=None):
 
-        def get_sql(item, db_module):
+        def get_sql(item, safe, db_module):
+            priv = None
+            if safe and item.session:
+                priv = item.session.find_privileges(self)
             if item.master:
                 item._master_id_field.set_data(item.master.ID)
                 item._master_rec_id_field.set_data(item.master._primary_key_field.value)
             if item.record_status == common.RECORD_INSERTED:
                 if not item.master:
                     if priv and not priv['can_create']:
-                        raise Exception, self.task.lang['cant_create'] % self.item_caption
+                        raise Exception(self.task.lang['cant_create'] % self.item_caption)
                 sql, param = item.insert_sql(db_module)
             elif item.record_status == common.RECORD_MODIFIED:
                 if not item.master:
                     if priv and not priv['can_edit']:
-                        raise Exception, self.task.lang['cant_edit'] % self.item_caption
+                        raise Exception(self.task.lang['cant_edit'] % self.item_caption)
                 sql, param = item.update_sql(db_module)
             elif item.record_status == common.RECORD_DETAILS_MODIFIED:
                 sql, param = '', None
             elif item.record_status == common.RECORD_DELETED:
                 if not item.master:
                     if priv and not priv['can_delete']:
-                        raise Exception, self.task.lang['cant_delete'] % self.item_caption
+                        raise Exception(self.task.lang['cant_delete'] % self.item_caption)
                 sql = item.delete_sql(db_module)
                 param = None
             else:
-                raise Exception, u'apply_sql - invalid %s record_status %s, record: %s' % (item.item_name, item.record_status, item._dataset[item.rec_no])
+                raise Exception('apply_sql - invalid %s record_status %s, record: %s' % \
+                    (item.item_name, item.record_status, item._dataset[item.rec_no]))
             primary_key_index = item.fields.index(item._primary_key_field)
             master_rec_id_index = None
             if item.master:
@@ -114,7 +116,7 @@ class SQL(object):
                 'log_id': item.get_rec_info()[common.REC_CHANGE_ID],
                 'master_rec_id_index': master_rec_id_index
                 }
-            h_sql, h_params, h_table_name = get_history_sql(item, user_info, db_module)
+            h_sql, h_params, h_table_name = get_history_sql(item, db_module)
             return sql, param, info, h_sql, h_params, h_table_name
 
         def delete_detail_sql(item, detail, db_module):
@@ -126,19 +128,24 @@ class SQL(object):
                 sql = 'DELETE FROM %s WHERE %s = %s AND %s = %s' % \
                     (detail.table_name, detail._master_id, item.ID, \
                     detail._master_rec_id, self._primary_key_field.value)
-            h_sql, h_params, h_table_name = get_history_sql(item, user_info, db_module)
+            h_sql, h_params, h_table_name = get_history_sql(item, db_module)
             return db_module.set_case(sql), None, None, h_sql, h_params, h_table_name
 
-        def get_history_sql(item, user_info, db_module):
+        def get_history_sql(item, db_module):
             h_sql = None
             h_params = None
             h_table_name = None
-            if item.task.history_item and item.keep_history or (item.master and item.master.keep_history):
+            if item.session:
+                user_info = item.session.user_info()
+            if item.task.history_item and item.keep_history:
                 h_table_name = item.task.history_item.table_name
                 changes = None
                 user = None
                 if user_info:
-                    user = user_info['user_name']
+                    try:
+                        user = user_info['user_name']
+                    except:
+                        pass
                 if item.record_status != common.RECORD_DELETED:
                     old_rec = item.get_rec_info()[3]
                     new_rec = item._dataset[item.rec_no]
@@ -174,11 +181,11 @@ class SQL(object):
                     (item.task.history_item.table_name, fields)) + '(' + values + ')'
             return h_sql, h_params, h_table_name
 
-        def generate_sql(item, db_module, result):
+        def generate_sql(item, safe, db_module, result):
             ID, sql = result
             for it in item:
                 details = []
-                sql.append((get_sql(it, db_module), details))
+                sql.append((get_sql(it, safe, db_module), details))
                 for detail in item.details:
                     detail_sql = []
                     detail_result = (str(detail.ID), detail_sql)
@@ -186,23 +193,29 @@ class SQL(object):
                     if item.record_status == common.RECORD_DELETED:
                         detail_sql.append((delete_detail_sql(item, detail, db_module), []))
                     else:
-                        generate_sql(detail, db_module, detail_result)
+                        generate_sql(detail, safe, db_module, detail_result)
 
         if db_module is None:
             db_module = self.task.db_module
         result = (self.ID, [])
-        generate_sql(self, db_module, result)
+        generate_sql(self, safe, db_module, result)
         return {'delta': result}
 
     def table_alias(self):
         return '"%s"' % self.table_name
 
-    def ref_table_alias(self, field):
+    def lookup_table_alias(self, field):
         if field.master_field:
             return '%s_%d' % (field.lookup_item.table_name, field.master_field.ID)
         else:
             return '%s_%d' % (field.lookup_item.table_name, field.ID)
 
+    def lookup_table_alias1(self, field):
+        return self.lookup_table_alias(field) + '_' + field.lookup_field
+
+    def lookup_table_alias2(self, field):
+        return self.lookup_table_alias1(field) + '_' + field.lookup_field1
+
     def fields_clause(self, query, fields, db_module=None):
         if db_module is None:
             db_module = self.task.db_module
@@ -227,13 +240,30 @@ class SQL(object):
         if query['__expanded']:
             for field in fields:
                 if field.lookup_item:
-                    field_sql = '%s."%s" %s %s_LOOKUP' % \
-                    (self.ref_table_alias(field), field.lookup_field, db_module.FIELD_AS, field.field_name)
-                    if funcs:
-                        func = functions.get(field.field_name.upper())
-                        if func:
-                            field_sql = '%s(%s."%s") %s %s_LOOKUP' % \
-                            (func, self.ref_table_alias(field), field.lookup_field, db_module.FIELD_AS, field.field_name)
+                    if field.lookup_field2:
+                        field_sql = '%s."%s" %s %s_LOOKUP' % \
+                        (self.lookup_table_alias2(field), field.lookup_field2, db_module.FIELD_AS, field.field_name)
+                        if funcs:
+                            func = functions.get(field.field_name.upper())
+                            if func:
+                                field_sql = '%s(%s."%s") %s %s_LOOKUP' % \
+                                (func, self.lookup_table_alias2(field), field.lookup_field2, db_module.FIELD_AS, field.field_name)
+                    elif field.lookup_field1:
+                        field_sql = '%s."%s" %s %s_LOOKUP' % \
+                        (self.lookup_table_alias1(field), field.lookup_field1, db_module.FIELD_AS, field.field_name)
+                        if funcs:
+                            func = functions.get(field.field_name.upper())
+                            if func:
+                                field_sql = '%s(%s."%s") %s %s_LOOKUP' % \
+                                (func, self.lookup_table_alias1(field), field.lookup_field1, db_module.FIELD_AS, field.field_name)
+                    else:
+                        field_sql = '%s."%s" %s %s_LOOKUP' % \
+                        (self.lookup_table_alias(field), field.lookup_field, db_module.FIELD_AS, field.field_name)
+                        if funcs:
+                            func = functions.get(field.field_name.upper())
+                            if func:
+                                field_sql = '%s(%s."%s") %s %s_LOOKUP' % \
+                                (func, self.lookup_table_alias(field), field.lookup_field, db_module.FIELD_AS, field.field_name)
                     sql += field_sql + ', '
         sql = sql[:-2]
         return db_module.set_case(sql)
@@ -243,13 +273,35 @@ class SQL(object):
             db_module = self.task.db_module
         result = db_module.FROM % (self.table_name, self.table_alias())
         if query['__expanded']:
+            joins = {}
             for field in fields:
-                if field.lookup_item and not field.master_field:
-                    primary_key_field_name = field.lookup_item._primary_key
-                    result = '(' + result
-                    result += ' ' + db_module.LEFT_OUTER_JOIN % (field.lookup_item.table_name, self.ref_table_alias(field))
-                    result += ' ON %s."%s"' % (self.table_alias(), field.field_name)
-                    result += ' = %s."%s")'  % (self.ref_table_alias(field), primary_key_field_name)
+                if field.lookup_item:
+                    alias = self.lookup_table_alias(field)
+                    cur_field = field
+                    if field.master_field:
+                        cur_field = field.master_field
+                    if not joins.get(alias):
+                        primary_key_field_name = field.lookup_item._primary_key
+                        result += ' ' + db_module.LEFT_OUTER_JOIN % (field.lookup_item.table_name, self.lookup_table_alias(field))
+                        result += ' ON %s."%s"' % (self.table_alias(), cur_field.field_name)
+                        result += ' = %s."%s"'  % (self.lookup_table_alias(field), primary_key_field_name)
+                        joins[alias] = True
+                if field.lookup_item1:
+                    alias = self.lookup_table_alias1(field)
+                    if not joins.get(alias):
+                        primary_key_field_name = field.lookup_item1._primary_key
+                        result += ' ' + db_module.LEFT_OUTER_JOIN % (field.lookup_item1.table_name, self.lookup_table_alias1(field))
+                        result += ' ON %s."%s"' % (self.lookup_table_alias(field), field.lookup_field)
+                        result += ' = %s."%s"'  % (self.lookup_table_alias1(field), primary_key_field_name)
+                        joins[alias] = True
+                if field.lookup_item2:
+                    alias = self.lookup_table_alias2(field)
+                    if not joins.get(alias):
+                        primary_key_field_name = field.lookup_item2._primary_key
+                        result += ' ' + db_module.LEFT_OUTER_JOIN % (field.lookup_item2.table_name, self.lookup_table_alias2(field))
+                        result += ' ON %s."%s"' % (self.lookup_table_alias1(field), field.lookup_field1)
+                        result += ' = %s."%s"'  % (self.lookup_table_alias2(field), primary_key_field_name)
+                        joins[alias] = True
         return db_module.set_case(result)
 
     def where_clause(self, query, db_module=None):
@@ -342,7 +394,10 @@ class SQL(object):
                 if filter_type in [common.FILTER_CONTAINS, common.FILTER_STARTWITH, common.FILTER_ENDWITH]:
                     value, esc_found = escape_search(value, esc_char)
                     if field.lookup_item:
-                        cond_field_name = db_module.set_case('%s."%s"' % (self.ref_table_alias(field), field.lookup_field))
+                        if field.lookup_item1:
+                            cond_field_name = db_module.set_case('%s."%s"' % (self.lookup_table_alias1(field), field.lookup_field1))
+                        else:
+                            cond_field_name = db_module.set_case('%s."%s"' % (self.lookup_table_alias(field), field.lookup_field))
                     if filter_type == common.FILTER_CONTAINS:
                         value = '%' + value + '%'
                     elif filter_type == common.FILTER_STARTWITH:
@@ -364,7 +419,7 @@ class SQL(object):
                 elif filter_sign == '<>':
                     sql = '(' + sql + ' AND %s IS NOT NULL)' % cond_field_name
                 else:
-                    raise Exception, 'sql.py where_clause method: boolen field condition may give ambiguious results.'
+                    raise Exception('sql.py where_clause method: boolen field condition may give ambiguious results.')
             return sql
 
         if db_module is None:
@@ -435,8 +490,6 @@ class SQL(object):
         if result:
             result = result[:-2]
             result = ' ORDER BY ' + result
-        elif query['__limit']:
-            result = ' ORDER BY %s."%s"' % (self.table_alias(), self._primary_key)
         return db_module.set_case(result)
 
     def get_select_statement(self, query, db_module=None):
@@ -456,9 +509,8 @@ class SQL(object):
                 self.order_clause(query, db_module),
                 fields)
             return sql
-        except Exception, e:
-            print self.item_name
-            print traceback.format_exc()
+        except Exception as e:
+            traceback.print_exc()
             raise
 
     def get_record_count_query(self, query, db_module=None):
@@ -490,7 +542,7 @@ class SQL(object):
         db_module = db_modules.get_db_module(db_type)
         result = db_module.create_table_sql(table_name, fields, foreign_fields)
         for i, s in enumerate(result):
-            print result[i]
+            print(result[i])
         return result
 
     def delete_table_sql(self, db_type):
@@ -498,7 +550,7 @@ class SQL(object):
         db_module = db_modules.get_db_module(db_type)
         result = db_module.delete_table_sql(table_name)
         for i, s in enumerate(result):
-            print result[i]
+            print(result[i])
         return result
 
     def recreate_table_sql(self, db_type, old_fields, new_fields, fk_delta=None):
@@ -582,7 +634,7 @@ class SQL(object):
     def change_table_sql(self, db_type, old_fields, new_fields):
 
         def recreate(comp):
-            for key, (old_field, new_field) in comp.items():
+            for key, (old_field, new_field) in comp.iteritems():
                 if old_field and new_field:
                     if old_field['field_name'] != new_field['field_name']:
                         return True
@@ -608,10 +660,10 @@ class SQL(object):
         if db_type == db_modules.SQLITE and recreate(comp):
             result += self.recreate_table_sql(db_type, old_fields, new_fields)
         else:
-            for key, (old_field, new_field) in comp.items():
+            for key, (old_field, new_field) in comp.iteritems():
                 if old_field and not new_field and db_type != db_modules.SQLITE:
                     result.append(db_module.del_field_sql(table_name, old_field))
-            for key, (old_field, new_field) in comp.items():
+            for key, (old_field, new_field) in comp.iteritems():
                 if old_field and new_field and db_type != db_modules.SQLITE:
                     if (old_field['field_name'] != new_field['field_name']) or \
                         (db_module.FIELD_TYPES[old_field['data_type']] != db_module.FIELD_TYPES[new_field['data_type']]) or \
@@ -622,11 +674,11 @@ class SQL(object):
                             result += sql
                         else:
                             result.append()
-            for key, (old_field, new_field) in comp.items():
+            for key, (old_field, new_field) in comp.iteritems():
                 if not old_field and new_field:
                     result.append(db_module.add_field_sql(table_name, new_field))
         for i, s in enumerate(result):
-            print result[i]
+            print(result[i])
         return result
 
     def create_index_sql(self, db_type, table_name, fields=None, new_fields=None, foreign_key_dict=None):
@@ -677,7 +729,7 @@ class SQL(object):
             else:
                 field_str = '"' + '", "'.join(fields) + '"'
             sql = db_module.create_index_sql(index_name, table_name, unique, field_str, desc)
-        print sql
+        print(sql)
         return sql
 
     def delete_index_sql(self, db_type):
@@ -688,5 +740,5 @@ class SQL(object):
             sql = db_module.delete_foreign_index(table_name, index_name)
         else:
             sql = db_module.delete_index(table_name, index_name)
-        print sql
+        print(sql)
         return sql
